package OIDC::Lite;

use strict;
use warnings;
use 5.008_001;

our $VERSION = '0.01';

1;
__END__

=head1 NAME

OIDC::Lite - OpenID Connect Library

=head1 DESCRIPTION

OpenID Connect Library

=head1 AUTHOR

Ryo Ito E<lt>ritou.06@gmail.comE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2012 by Ryo Ito

=head1 LICENSE

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
